#include <stdio.h>
#include <cstdint>
#include <vector>
#include <stdexcept>
// For reading FIFO
#include <fcntl.h>
#include <unistd.h>

// Local files
#include "utility/bitmap.hpp"
#include "utility/buffer.hpp"
#include "signal_processing/fft.hpp"
#include "ergodox.hpp"
#include "commands.h"


// TODO:
// stereo support (and maybe different bitrates?)
// Optimization: don't copy filename in buffer?
// understand consumption rate
// understand BIN2HZ
// understand values and sizes (DFT_TOTAL, DFT_NONZERO, etc)
//   note that wave and spectrum have different sizes
//
// MPD interface
// hid interface to keyboard
// beat detection


void draw_spectrum_bitmap(
	const std::vector<size_t>& waveform,
	Bitmap& bitmap
) {
	for (size_t x = 0; x < waveform.size(); x++) {
		for (size_t y = 0; y < waveform[x]; y++) {
			bitmap.setpixel(bitmap.get_height() - y - 1, x, 0xFF, 0x00, 0x00);
		}
	}
}



// How many keys in a column * resolution per key.
// this MUST fit inside a uint8_t.
const uint8_t kb_resolution = 5 * 50;

// How many resolution steps to skip at the top and bottom.
const size_t bottom_skip = 25;
const size_t top_skip = 50;


const size_t width = 10;
const size_t height = bottom_skip + kb_resolution + top_skip;


int main(int argc, char *argv[]) {

	// buffer size for waveform:
	// (44100 / fps * 10), make 10 bigger for slower scrolling
	//
	// Double both buffer sizes if stereo

	// FFT generator
	FFT_Visualizer fft = FFT_Visualizer(
		width, height,
		100, 5000
	);

	// Audio buffer
	Buffer buf = Buffer(
		"/tmp/mpd.fifo",
		44100 / 2, // Keep 500ms of data in buffer
		fft.compute_buffer_output_size()
	);

	// HID interface wrapper
	Ergodox Dox = Ergodox::init(
		0x3297,
		0x4976,
		0x61,
		0xFF60
	);

	// Write buffer
	uint8_t hid_buf[12];

	while (1) {


		if (Dox.get_animation_mode() == 0x02) {
			buf.update();
			fft.update(buf);

			hid_buf[0] = CMD_ANIM_DATA_fft;
			for (size_t i = 0; i < 10; i++) {
				// Get height from fft, apply bottom_skip
				ssize_t h = fft.get_output()[i] - bottom_skip;

				// Enforce max and min
				// max implicitly enforces top_skip
				h = h>kb_resolution ? kb_resolution : h;
				h = h<0 ? 0 : h;

				hid_buf[i + 1] = h;
			}
			Dox.write(CMD_ANIM_DATA, hid_buf, 13);
		}

		// Read a packet if there is a packet to read
		if (Dox.read()) {
			uint8_t cmd = Dox.read_buf[0];

			switch(cmd) {
				case CMD_SEND_STATE:
					break;
			}
		}
	}

	return 0;
}